function love.load()
    -- some configs
    number_of_columns = 4 -- x direction
    number_of_rows = 4 -- y direction
    
    active_piece = nil
    
    snapping_distance = 10 -- in pixel
	--C: Adding a new variable (this should not be global) to store all the pieces.
    canvas = {}
end

function love.draw()
    if video == nil then
        love.graphics.print("Please drag and drop an ogg theora video in here.", 241, 293)
    else
        love.graphics.setColor(0,0,192)
        love.graphics.rectangle("fill", 0, 0, 800, 600)
        love.graphics.setColor(255, 255, 255)
        love.graphics.rectangle("fill", (800-video:getWidth())/2, (600-video:getHeight())/2, video:getWidth(), video:getHeight())
        
		--C: Removed old variable canvae and the video is now drawn to a big canvas
		--C: This will allow us to render it without having to draw the video again and again for each peice
		--C: Instead we will draw the canvas which, I think, is faster than drawing a video
        love.graphics.setCanvas(canvas.big)
        love.graphics.draw(video, 0,0)
		--C: We will use the variable "ci" to count which piece are currently in
        local ci = 0
        for i=1,number_of_columns do
            for j=1, number_of_rows do
                ci = ci+1
				--C: Now we draw the big canvas to the corresponding piece
                love.graphics.setCanvas(canvas[ci])
                    love.graphics.draw(canvas.big, (1-i)*width, (1-j)*height)
                love.graphics.setCanvas()
            end
        end
        
		--C: Replace "canvae" with "canvas"
        for i=1, #canvas do
            if i == active_piece then
                -- add red border
                love.graphics.setColor(255,0,0)
                love.graphics.rectangle("line", pos[i][1]-1, pos[i][2]-1, width+2, height+2)
                love.graphics.setColor(255,255,255)
            end
            love.graphics.draw(canvas[i], pos[i][1], pos[i][2])
        end
    end
end

function love.filedropped(file)
    -- crashes if it cannot open the video - whatever
    video = love.graphics.newVideo(file)
    video:play()
    
    width = video:getWidth()/number_of_columns
    height = video:getHeight()/number_of_rows
    
    -- generate random positions
    pos = {}
    for i=1,number_of_columns*number_of_rows do
        pos[i] = {love.math.random(0, 800-width), love.math.random(0, 600-height)}
    end
    --C: Creating the canvas in here will increase performance due to the fact that canvases are expensive to create
    --C: Now we won't need to create them every frame
    for i=1,number_of_columns do
        for j=1, number_of_rows do
            table.insert(canvas,love.graphics.newCanvas(width, height))
        end
    end
    canvas.big = love.graphics.newCanvas(video:getWidth(),video:getHeight())

end

function love.keypressed(key)
    if key=="escape" then
        love.event.quit()
    end
end

function love.mousepressed(x, y, button)
    if pos ~= nil then
        -- get the top most video
        for i=number_of_columns*number_of_rows, 1, -1 do
            if x>=pos[i][1] and x<=pos[i][1] + width and y>=pos[i][2] and y<=pos[i][2] + height then
                -- got it!
                active_piece = i
                break
            end
        end
    end
end

function love.mousereleased()
    -- snap it in place when it is nearly where a piece can be
	--C: Adding this line will stop your game from crashing when the player clicks on a space with no pieces
	--C: This works since "active_piece" will only be set when the player presses a piece, if not, "active_canvas" will be set to nil
    if active_piece then
        for i=1,number_of_columns do
            for j=1, number_of_rows do
                if pos[active_piece][1] >= (800-video:getWidth())/2 + (i-1)*width - snapping_distance
                    and pos[active_piece][1] <= (800-video:getWidth())/2 + (i-1)*width + snapping_distance
                    and pos[active_piece][2] >= (600-video:getHeight())/2 + (j-1)*height - snapping_distance
                    and pos[active_piece][2] <= (600-video:getHeight())/2 + (j-1)*height + snapping_distance then
                    
                    pos[active_piece][1] = (800-video:getWidth())/2 + (i-1)*width
                    pos[active_piece][2] = (600-video:getHeight())/2 + (j-1)*height
                end
            end
        end
    end
	--C: The won check will also be inside an if caluse that check if the video has been loaded
	--C: This fixes an error that happened when the player tried to click on the screen before loading the video
	--C: The error happened because the game was checking if the player had won before creating all the necessary variables that are created when the video is loaded
	--C: Variables such as "pos" are only created after the video loads and won() needs these variables or else it will be buggy, since they are set to nil prior the the loading of the video
	if video then
    	if won() then
            love.mousepressed = nil
        end
	end
    
    active_piece = nil
    
end

function love.mousemoved(x, y, dx, dy)
    if active_piece ~= nil and pos ~= nil then
        -- move the piece with the mouse
        pos[active_piece][1] = pos[active_piece][1] + dx
        pos[active_piece][2] = pos[active_piece][2] + dy
    end
end

function won()
    local result = true
    
    for i=1,number_of_columns do
        for j=1, number_of_rows do
            if (pos[(i-1)*number_of_columns+j][1] ~= (800-video:getWidth())/2 + (i-1)*width) and (pos[(j-1)*number_of_rows+i][2] ~= (600-video:getHeight())/2 + (j-1)*height) then
                result = false
            end
        end
    end
    
    return result
end

function love.update(dt)
    if video ~= nil and not video:isPlaying() then
        video:rewind()
        video:play()
    end
end
